import { ref, reactive, computed, toRefs } from "vue";
import { Table } from "@/hooks/interface";
import { ColumnProps } from "@/components/LjVxeTable/interface";
import { ALLOW_EDIT_STATE } from "@/config/index";
import { CommonDynamicSelect } from "@/api/modules/common";
import { useUserStore } from "@/stores/modules/user";

interface defaultState {
  /**
   * @description 单据当前状态
   */
  orderStatus: string;
  LjDetailRef: any;
}
/**
 * @description 表格多选数据操作
 * @param {String} rowKey 当表格可以多选时，所指定的 id
 * */
export const useHooks = (t?: any) => {
  const state = reactive<defaultState>({
    orderStatus: "",
    LjDetailRef: ""
  });

  const funcMoneyrateChange = async (val: any, data) => {
    let newParams = {
      dsname: "_Mapper_deptid",
      queryparams: {
        arg_deptid: data.deptid
      }
    };
    let res = await CommonDynamicSelect(newParams);

    let _discount = 1;
    let _moneyrate = 1;
    if (res.datatable?.length > 0) {
      _discount = res.datatable[0].discount;
      _moneyrate = res.datatable[0].moneyrate;
    }
    data.moneyrate = _moneyrate;
    data.discount = _discount;
  };

  // 表格配置项
  const columns: ColumnProps<any>[] = [
    { type: "checkbox", width: 50, fixed: "left" },
    { title: "#", field: "pid", fixed: "left", width: 80 },
    {
      field: "deptid",
      title: "部门",
      search: {
        el: "select",
        key: "arg_deptid",
        props: {
          filterable: true
        },
        order: 1
      },
      basicinfo: {
        order: 1,
        group: "单据信息"
      }
    },
    {
      field: "mattressname",
      title: "床垫名称",
      align: "left",
      table: "u_mattress",
      search: {
        el: "input",
        key: "arg_search",
        titleKey: "searchKw",
        props: {
          placeholder: "唯一码/编码/名称"
        },
        order: 0
      },
      basicinfo: {
        el: "input",
        editable: ALLOW_EDIT_STATE
      }
    },
    {
      field: "flag",
      title: "财务审核",
      table: "u_mattress",
      search: {
        el: "select",
        key: "status_flag",
        titleKey: "status",
        props: {
          clearable: true,
          placeholder: "全部"
        },
        order: 2,
        render: scope => {
          let _keys = Object.keys(scope);
          // let _data = _keys.includes("row") ? scope.row : _keys.includes("searchParam") ? scope.searchParam : scope;
          return (
            <el-select v-model={scope.searchParam.status_flag} placeholder="全部" clearable>
              <el-option label="待下单" value="1" />
              <el-option label="已下单" value="2" />
              <el-option label="待财务审核" value="3" />
              <el-option label="已财务审核" value="4" />
            </el-select>
          );
        }
      }
    },
    {
      field: "dept_profitrate",
      title: "部门让利点数",
      basicinfo: {
        order: 2,
        group: "单据信息"
      }
    },
    {
      field: "fees_dscrp",
      title: "额外费用说明",
      basicinfo: {
        el: "input",
        props: { type: "textarea", rows: 2 },
        row: 2
      }
    },
    {
      field: "moneyrate",
      title: "汇率",
      basicinfo: {
        el: "select",
        render: (scope: any) => {
          let _keys = Object.keys(scope);
          console.log("if_moneyrate scope :>> ", scope);
          let optionRender = [];
          scope.enum.map(item => {
            optionRender.push(<el-option label={item.label} value={item.value} />);
          });
          let slotprefix = {
            prefix: () => {
              return <>{scope.searchParam.moneyrate}</>;
            }
          };

          return (
            <el-select
              v-model={scope.searchParam.if_moneyrate}
              v-slots={slotprefix}
              class="select-text-right"
              onChange={val => funcMoneyrateChange(val, scope.searchParam)}
            >
              {optionRender}
            </el-select>
          );
        }
      }
    },
    {
      field: "foreign_cost",
      table: "u_mattress",
      title: "外币价",
      basicinfo: {
        titleKey: "bi_foreign_cost"
      }
    },
    {
      field: "nottax_dept_cost",
      table: "u_mattress",
      title: "部门不含税价",
      basicinfo: {
        titleKey: "bi_nottax_dept_cost"
      }
    },
    {
      field: "if_bcp_type",
      title: "半成品",
      basicinfo: {
        labelHidden: true,
        render: (scope: any) => {
          return (
            <>
              <el-checkbox v-model={scope.searchParam.if_bcp_type} label="半成品" class="mr-8" />
              <el-checkbox v-model={scope.searchParam.if_zhedie_type} label="折叠款" />
            </>
          );
        }
      }
    },
    {
      field: "if_m_chai",
      title: "面拆",
      basicinfo: {
        labelHidden: true,
        render: (scope: any) => {
          return (
            <>
              <el-checkbox v-model={scope.searchParam.if_m_chai} label="面拆" class="mr-8" />
              <el-checkbox v-model={scope.searchParam.if_z_chai} label="中拆" class="mr-8" />
              <el-checkbox v-model={scope.searchParam.if_d_chai} label="底拆" />
            </>
          );
        }
      }
    }
  ];

  const columnsMx: ColumnProps<any>[] = [
    { title: "#", type: "seq", fixed: "left", width: 80 },
    {
      title: "计算成功?",
      field: "if_success",
      datatype: "checkbox"
    },
    {
      title: "+",
      field: "addmx",
      width: 80
    },
    {
      title: "-",
      field: "delmx",
      width: 80
    },
    {
      title: "位置",
      field: "chastr"
    },
    {
      title: "公式名",
      field: "formulakind",
      table: "u_mattress_formula"
    },
    {
      title: "按物料？",
      field: "if_mtrl",
      table: "u_mattress_formula"
    },
    {
      title: "厚度cm",
      field: "thickness"
    },
    {
      title: "物料",
      field: "mtrlid"
    },
    {
      title: "数量/长度",
      field: "qty"
    },
    {
      title: "物料单价",
      field: "price"
    },
    {
      title: "单位",
      field: "priceunit"
    },
    {
      title: "收缩率",
      field: "shrinkage"
    },
    {
      title: "克重",
      field: "gram_weight"
    },
    {
      title: "幅宽",
      field: "cloth_width"
    },
    {
      title: "固定厚度?",
      field: "if_inputqty"
    },
    {
      title: "按面积单价？",
      field: "if_areaprice"
    },
    {
      title: "成本金额",
      field: "costamt"
    },
    {
      title: "金额文本公式",
      field: "formula"
    },
    {
      title: "金额数字公式",
      field: "replace_formula"
    },
    {
      title: "清单用量",
      field: "useqty"
    },
    {
      title: "用量文本公式",
      field: "useformula"
    },
    {
      title: "用量数字公式",
      field: "replace_useformula"
    },
    {
      title: "工艺填写说明",
      field: "gydscrp"
    }
  ];

  const summaryData = ref<any>([
    {
      label: "总成本",
      field: "total_cost",
      formula:
        "if([折叠款] = 1, ([总材料成本]*[大小单]+[款式费用]+[边带费用]+[额外费用]+[制造费用]) * 2, [总材料成本]*[大小单]+[款式费用]+[边带费用]+[额外费用]+[制造费用])",
      power: () => {
        const { userInfo } = useUserStore();
        if (userInfo.empid != 0 && userInfo.usermode != 0) {
          return false;
        } else {
          return true;
        }
      }
    },
    {
      label: "不含税出厂价",
      field: "nottax_factory_cost",
      formula: "[总成本] * ([工厂利润率] + [利润率点数] + [布套点数] + [拆装点数] + [海绵点数]) * [管理费点]"
    },
    {
      label: "部门售价",
      field: "nottax_dept_cost",
      formula: "[dijia_cost] / (1 - ([佣金点数] - 1)) * [额外点数] + [fob]"
    },
    {
      label: "税金",
      field: "taxes",
      formula: "([部门售价] * ([税率] - 1)) + [佣金] * 0.05"
    },
    {
      label: "部门含税价",
      field: "dept_cost",
      formula: "([部门售价] + [佣金] * 0.06) * [税率] * if([fob] = 0, 1, [折扣率])"
    },
    {
      label: "外币价",
      field: "foreign_cost",
      formula: "if([汇率] <> 0, [部门含税价] / [汇率], 0)"
    },
    {
      label: "底价 ",
      field: "dijia_cost_cp",
      formula: "if(([dept_profitrate] + [dept_profitrate_rangli] / 100) <> 0, [dijia_cost] ,0 )"
    },
    {
      label: "佣金",
      field: "commission_cp",
      formula: "[dijia_cost] / (1 - ([佣金点数] - 1 )) - [dijia_cost]"
    },
    {
      label: "FOB",
      field: "fob",
      formula: "[fob]"
    }
  ]);

  const fieldParams = ref<any>([
    {
      label: "总成本",
      field: "total_cost"
    },
    {
      label: "不含税出厂价",
      field: "nottax_factory_cost"
    },
    {
      label: "部门售价",
      field: "nottax_dept_cost"
    },
    {
      label: "税金",
      field: "taxes"
    },
    {
      label: "部门含税价",
      field: "dept_cost"
    },
    {
      label: "外币价",
      field: "foreign_cost"
    },
    {
      label: "总材料成本",
      field: "total_material_cost"
    },
    {
      label: "大小单",
      field: "dannum_rate"
    },
    {
      label: "款式费用",
      field: "hrcost"
    },
    {
      label: "边带费用",
      field: "biandaicost"
    },
    {
      label: "额外费用",
      field: "extras_cost"
    },
    {
      label: "制造费用",
      field: "zhizao_amt"
    },
    {
      label: "折叠款",
      field: "if_zhedie_type"
    },
    {
      label: "管理费点",
      field: "guanli_rate"
    },
    {
      label: "工厂利润率",
      field: "profitrate"
    },
    {
      label: "利润率点数",
      field: "profitrate_point"
    },
    {
      label: "布套点数",
      field: "butao_point"
    },
    {
      label: "拆装点数",
      field: "chaizhuang_point"
    },
    {
      label: "海绵点数",
      field: "haimian_point"
    },
    {
      label: "部门利润率",
      field: "dept_profitrate"
    },
    {
      label: "部门让利点",
      field: "dept_profitrate_rangli"
    },
    {
      label: "额外点数",
      field: "other_rate"
    },
    {
      label: "佣金点数",
      field: "commission"
    },
    {
      label: "税率",
      field: "taxrate"
    },
    {
      label: "折扣率",
      field: "discount"
    },
    {
      label: "汇率",
      field: "moneyrate"
    },
    {
      label: "底价",
      field: "dijia_cost"
    }
  ]);

  const columnsMxBednet: ColumnProps<any>[] = [];
  const columnsMxCushions: ColumnProps<any>[] = [];
  const columnsMxAccessories: ColumnProps<any>[] = [];
  const columnsMxInnerClothLayer: ColumnProps<any>[] = [];

  return {
    ...toRefs(state),
    columns,
    columnsMx,
    columnsMxBednet,
    columnsMxCushions,
    columnsMxAccessories,
    columnsMxInnerClothLayer,
    summaryData,
    fieldParams
  };
};
